@@ -12,6 +12,7 @@ Copyright 2021 The Microsoft DeepSpeed Team
 # This source code is licensed under the BSD license found in the
 # LICENSE file in the root directory of this source tree.
 
+from deepspeed.utils.timer import ThroughputTimer, SynchronizedWallClockTimer
 from typing import Callable, Dict, TYPE_CHECKING, Any, Optional, Tuple, Union, cast
 
 import time
@@ -95,14 +96,50 @@ import torch.nn.functional as F
 
 import math
 
+# einsum rewrites are on par or more performant
+# switch can be bubbled up in future
+USE_EINSUM = True
+
+
+def einsum(rule, a, b):
+    if USE_EINSUM:
+        return torch.einsum(rule, a, b)
+    elif rule == 's,se->se':
+        return a.reshape(a.shape[0], -1) * b
+    elif rule == 'se,sc->sec':
+        return a.unsqueeze(2) * b.unsqueeze(1)
+    elif rule == 'se,se->s':
+        return torch.bmm(a.unsqueeze(1), b.unsqueeze(2)).reshape(-1)
+    elif rule == 'sec,sm->ecm':
+        s = a.shape[0]
+        e = a.shape[1]
+        c = a.shape[2]
+        m = b.shape[1]
+        return torch.matmul(a.reshape(s, -1).t(), b).reshape(e, c, m)
+    elif rule == 'sec,ecm->sm':
+        return torch.matmul(a.reshape(a.shape[0], -1), b.reshape(-1, b.shape[-1]))
+    elif rule == 'ks,ksm->sm':
+        k = b.shape[0]
+        s = b.shape[1]
+        m = b.shape[2]
+        # [k, s] -> [s, k] -> [s, 1, k]
+        a = a.t().unsqueeze(1)
+        # [k,s,m] -> [k, sm] -> [sm, k] -> [s, m, k]
+        b = b.reshape(k, -1).t().reshape(s, m, k)
+        # bmm([s, 1, k], [s, m, k]^t) -> [s, m, 1]
+        return torch.bmm(a, b.transpose(1, 2)).squeeze(2)
+    else:
+        return torch.einsum(rule, a, b)
+
 
 def top1gating(logits: torch.Tensor,
                capacity_factor: float,
                min_capacity: int,
                used_token: torch.Tensor = None,
-               noisy_gate_policy: Optional[str] = None) -> Tuple[Tensor,
-                                                                 Tensor,
-                                                                 Tensor]:
+               noisy_gate_policy: Optional[str] = None,
+               drop_tokens: bool = True) -> Tuple[Tensor,
+                                                  Tensor,
+                                                  Tensor]:
     """Implements Top1Gating on logits."""
     if noisy_gate_policy == 'RSample':
         logits_w_noise = logits + gumbel_rsample(logits.shape, device=logits.device)
@@ -110,8 +147,8 @@ def top1gating(logits: torch.Tensor,
     gates = F.softmax(logits, dim=1)
 
     # gates has shape of SE
-    num_tokens = gates.shape[0]
-    num_experts = gates.shape[1]
+    num_tokens = int(gates.shape[0])
+    num_experts = int(gates.shape[1])
     # round-up
     capacity = math.ceil((num_tokens / num_experts) * capacity_factor)
     if capacity < min_capacity:
@@ -126,11 +163,17 @@ def top1gating(logits: torch.Tensor,
 
     # mask only used tokens
     if used_token is not None:
-        mask1 = torch.einsum("s,se->se", used_token, mask1)
+        mask1 = einsum("s,se->se", used_token, mask1)
 
     # gating decisions
     exp_counts = torch.sum(mask1, dim=0).detach().to('cpu')
 
+    # if we don't want to drop any tokens
+    if not drop_tokens:
+        new_capacity = torch.max(exp_counts).to(logits.device)
+        dist.all_reduce(new_capacity, op=dist.ReduceOp.MAX, group=dist.group.WORLD)
+        capacity = new_capacity
+
     # Compute l_aux
     me = torch.mean(gates, dim=0)
     ce = torch.mean(mask1.float(), dim=0)
@@ -164,7 +207,8 @@ def top1gating(logits: torch.Tensor,
     gates = gates * mask1_float
 
     locations1_sc = F.one_hot(locations1_s, num_classes=capacity).float()
-    combine_weights = torch.einsum("se,sc->sec", gates, locations1_sc)
+    combine_weights = einsum("se,sc->sec", gates, locations1_sc)
+
     dispatch_mask = combine_weights.bool()
 
     return l_aux, combine_weights, dispatch_mask, exp_counts
@@ -180,8 +224,8 @@ def top2gating(logits: torch.Tensor,
     gates = F.softmax(logits, dim=1)
 
     # gates has shape of SE
-    num_tokens = gates.shape[0]
-    num_experts = gates.shape[1]
+    num_tokens = int(gates.shape[0])
+    num_experts = int(gates.shape[1])
     # capacity = (2 * num_tokens // num_experts) * capacity_factor
     # round-up
     capacity = math.ceil((2 * num_tokens / num_experts) * capacity_factor)
@@ -223,8 +267,8 @@ def top2gating(logits: torch.Tensor,
     # Normalize gate probabilities
     mask1_float = mask1.float()
     mask2_float = mask2.float()
-    gates1_s = torch.einsum("se,se->s", gates, mask1_float)
-    gates2_s = torch.einsum("se,se->s", gates, mask2_float)
+    gates1_s = einsum("se,se->s", gates, mask1_float)
+    gates2_s = einsum("se,se->s", gates, mask2_float)
     denom_s = gates1_s + gates2_s
     # Avoid divide-by-zero
     denom_s = torch.clamp(denom_s, min=torch.finfo(denom_s.dtype).eps)
@@ -232,12 +276,12 @@ def top2gating(logits: torch.Tensor,
     gates2_s /= denom_s
 
     # Calculate combine_weights and dispatch_mask
-    gates1 = torch.einsum("s,se->se", gates1_s, mask1_float)
-    gates2 = torch.einsum("s,se->se", gates2_s, mask2_float)
+    gates1 = einsum("s,se->se", gates1_s, mask1_float)
+    gates2 = einsum("s,se->se", gates2_s, mask2_float)
     locations1_sc = F.one_hot(locations1_s, num_classes=capacity).float()
     locations2_sc = F.one_hot(locations2_s, num_classes=capacity).float()
-    combine1_sec = torch.einsum("se,sc->sec", gates1, locations1_sc)
-    combine2_sec = torch.einsum("se,sc->sec", gates2, locations2_sc)
+    combine1_sec = einsum("se,sc->sec", gates1, locations1_sc)
+    combine2_sec = einsum("se,sc->sec", gates2, locations2_sc)
     combine_weights = combine1_sec + combine2_sec
     dispatch_mask = combine_weights.bool()
 
@@ -269,7 +313,8 @@ class TopKGate(torch.nn.Module):
                  capacity_factor: float = 1.0,
                  eval_capacity_factor: float = 1.0,
                  min_capacity: int = 4,
-                 noisy_gate_policy: Optional[str] = None) -> None:
+                 noisy_gate_policy: Optional[str] = None,
+                 drop_tokens: bool = True) -> None:
         super().__init__()
 
         # Only top-1 and top-2 are supported at the moment.
@@ -281,6 +326,10 @@ class TopKGate(torch.nn.Module):
         self.eval_capacity_factor = eval_capacity_factor
         self.min_capacity = min_capacity
         self.noisy_gate_policy = noisy_gate_policy
+        self.timers = SynchronizedWallClockTimer()
+        self.wall_clock_breakdown = False
+        self.gate_time = 0.0
+        self.drop_tokens = drop_tokens
 
     def forward(
         self,
@@ -289,6 +338,10 @@ class TopKGate(torch.nn.Module):
     ) -> Tuple[Tensor,
                Tensor,
                Tensor]:  # type: ignore
+
+        if self.wall_clock_breakdown:
+            self.timers('TopKGate').start()
+
         if self.wg.weight.dtype != torch.float32:
             self.wg = self.wg.float()
         input_fp32 = input.float()
@@ -296,18 +349,27 @@ class TopKGate(torch.nn.Module):
         if self.noisy_gate_policy == 'Jitter' and self.training:
             input_fp32 = multiplicative_jitter(input_fp32, device=input.device)
         logits = self.wg(input_fp32)
+
         if self.k == 1:
-            return top1gating(
+            gate_output = top1gating(
                 logits,
                 self.capacity_factor if self.training else self.eval_capacity_factor,
                 self.min_capacity,
                 used_token,
-                self.noisy_gate_policy if self.training else None)
+                self.noisy_gate_policy if self.training else None,
+                self.drop_tokens)
+
         else:
-            return top2gating(
+            gate_output = top2gating(
                 logits,
                 self.capacity_factor if self.training else self.eval_capacity_factor)
 
+        if self.wall_clock_breakdown:
+            self.timers('TopKGate').stop()
+            self.gate_time = self.timers('TopKGate').elapsed(reset=False) * 1000
+
+        return gate_output
+
 
 class MOELayer(Base):
     """MOELayer module which implements MixtureOfExperts as described in Gshard_.
@@ -337,41 +399,70 @@ class MOELayer(Base):
         self.group = group
         self.world_size = dist.get_world_size(group)
         self.num_local_experts = num_local_experts
+        self.time_falltoall = 0.0
+        self.time_salltoall = 0.0
+        self.time_moe = 0.0
+        self.timers = SynchronizedWallClockTimer()
+        self.wall_clock_breakdown = False
 
     def forward(self, *input: Tensor, **kwargs: Any) -> Tensor:
-        # assert len(input) == 1, "only single input Tensor supported"
-        # assert len(input[0].shape) == 3 or len(input[0].shape) == 2, "input Tensor must have dimensions: (s)equence, (t)oken, (m)odel"
-        # removed wrong assert
-        # assert input[0].shape[0] % len(self.experts) == 0, "num tokens must be order of number of local experts"
+
+        if self.wall_clock_breakdown:
+            self.timers('moe').start()
+
         # Implement Algorithm 2 from GShard paper.
         d_model = input[0].shape[-1]
+
         # Initial implementation -> Reshape into S tokens by dropping sequence dimension.
         # Reshape into G groups so that each group can distribute tokens equally
         # group_size = kwargs['group_size'] if 'group_size' in kwargs.keys() else 1
         reshaped_input = input[0].reshape(-1, d_model)
-        self.l_aux, combine_weights, dispatch_mask, self.exp_counts  = self.gate(reshaped_input, input[1])
-        dispatched_input = torch.einsum("sec,sm->ecm",
-                                        dispatch_mask.type_as(input[0]),
-                                        reshaped_input)
 
-        #print(f"alltoall called at rank:{dist.get_rank()} with dispatched_input shape:{dispatched_input.shape}")
-        a = time.perf_counter()
+        self.l_aux, combine_weights, dispatch_mask, self.exp_counts  = self.gate(reshaped_input, input[1])
+
+        dispatched_input = einsum("sec,sm->ecm",
+                                  dispatch_mask.type_as(input[0]),
+                                  reshaped_input)
+
+        if self.wall_clock_breakdown:
+            self.timers('falltoall').start()
+
         dispatched_input = _AllToAll.apply(self.group, dispatched_input)
-        b = time.perf_counter()
-        #print(f"alltoall took {b-a} seconds at rank:{dist.get_rank()}")
+
+        if self.wall_clock_breakdown:
+            self.timers('falltoall').stop()
+            self.time_falltoall = self.timers('falltoall').elapsed(reset=False) * 1000
+
         # Re-shape after all-to-all: ecm -> gecm
         dispatched_input = dispatched_input.reshape(self.world_size,
                                                     self.num_local_experts,
                                                     -1,
                                                     d_model)
-        #print(f"reshaped input after alltoall called at rank:{dist.get_rank()} is dispatched_input shape:{dispatched_input.shape}")
+
         expert_output = self.experts(dispatched_input)
+
+        if self.wall_clock_breakdown:
+            self.timers('salltoall').start()
+
         expert_output = _AllToAll.apply(self.group, expert_output)
+
+        if self.wall_clock_breakdown:
+            self.timers('salltoall').stop()
+            self.time_salltoall = self.timers('salltoall').elapsed(reset=False) * 1000
+
         # Re-shape back: gecm -> ecm
         expert_output = expert_output.reshape(self.world_size * self.num_local_experts,
                                               -1,
                                               d_model)
-        combined_output = torch.einsum("sec,ecm->sm",
-                                       combine_weights.type_as(input[0]),
-                                       expert_output)
-        return combined_output.reshape(input[0].shape)
+
+        combined_output = einsum("sec,ecm->sm",
+                                 combine_weights.type_as(input[0]),
+                                 expert_output)
+
+        a = combined_output.reshape(input[0].shape)
+
+        if self.wall_clock_breakdown:
+            self.timers('moe').stop()
+            self.time_moe = self.timers('moe').elapsed(reset=False) * 1000
+
+        return a
